
precision mediump float;
varying vec2 varyTexCoord;
varying vec2 varyTexCoordBackgr;
varying vec2 varyTexCoordNormal0;
varying vec2 varyTexCoordNormal1;
varying vec2 varyTexCoordNormal2;

uniform sampler2D texPhi;
uniform sampler2D texUV;
uniform sampler2D texBackgr;
uniform sampler2D texNormal;
uniform sampler2D texPaint;

uniform vec3 uWaterColor;
uniform vec4 uRippleStrAndBackgrScale; // x, y - ripple strengths speed dependent/independend; zw - background image scale
uniform vec3 uTransp_BlurWidth_SpecStrength;
uniform vec4 uGridSizeAndInv;

uniform vec3 uPaintColor0;
uniform vec3 uPaintColor1;
uniform vec3 uPaintColor2;

vec2 phiGradient()
{
	float halfPix = 0.5 / 800.0;
	float gradX = texture2D(texPhi, varyTexCoord + vec2(halfPix, 0.0)).r - texture2D(texPhi, varyTexCoord - vec2(halfPix, 0.0)).r;
	float gradY = texture2D(texPhi, varyTexCoord + vec2(0.0, halfPix)).r - texture2D(texPhi, varyTexCoord - vec2(0.0, halfPix)).r;
	return normalize(vec2(gradX, gradY));
}

float cubic(vec4 fourPoints, float x)
{
	float a3 = dot(vec4(-0.5, 1.5, -1.5, 0.5), fourPoints); // float a3 = -0.5*p0 + 1.5*p1 - 1.5*p2 + 0.5*p3;
	float a2 = dot(vec4(1.0, -2.5, 2.0, -0.5), fourPoints); // float a2 = p0 - 2.5*p1 + 2.0*p2 - 0.5*p3;
	float a1 = dot(vec4(-0.5, 0.0, 0.5, 0.0), fourPoints); // float a1 = -0.5*p0 + 0.5*p2;
	float a0 = fourPoints.y; // float a0 = p1;
	return (((a3 * x) + a2) * x + a1) * x + a0;
}

float phiSingleRowCubic(float baseX, float y, float xStep, float interpolant)
{
	float p0 = texture2D(texUV, vec2(baseX - xStep, 	   y)).b;
	float p1 = texture2D(texUV, vec2(baseX, 		 	   y)).b;
	float p2 = texture2D(texUV, vec2(baseX + xStep,	   y)).b;
	float p3 = texture2D(texUV, vec2(baseX + 2.0 * xStep, y)).b;
	
	return cubic(vec4(p0, p1, p2, p3), interpolant);
}

float phiBicubic()
{
	// should offset by half cell after scaling and before backscaling, if wanted to sample exactly at pixel (phi-cell-pixel!) centers
	// but the way it is now, cubic interp. samples are themselves results of linear interpolation, and it seems to look even better
	vec2 tex = varyTexCoord * uGridSizeAndInv.xy;
	vec2 texFloored = floor(tex);
	vec2 t = tex - texFloored;
	
	vec2 gridSizeInv = uGridSizeAndInv.zw;
	
	tex = (texFloored) * gridSizeInv;
	
	vec4 rows;
	rows.x = phiSingleRowCubic(tex.x, tex.y - gridSizeInv.y,		 gridSizeInv.x, t.x);
	rows.y = phiSingleRowCubic(tex.x, tex.y,			 			 gridSizeInv.x, t.x);
	rows.z = phiSingleRowCubic(tex.x, tex.y + gridSizeInv.y,		 gridSizeInv.x, t.x);
	rows.w = phiSingleRowCubic(tex.x, tex.y + 2.0 * gridSizeInv.y, gridSizeInv.x, t.x);
	return cubic(rows, t.y);
}

float phiBicubicOpt()
{
	vec2 tex = varyTexCoord * uGridSizeAndInv.xy;
	vec2 texFloored = floor(tex);
	vec2 t = tex - texFloored;
	
	vec2 gridSizeInv = uGridSizeAndInv.zw;
	
	tex = (texFloored/* + 0.25*/) * gridSizeInv; // <- clamping issue with this tricky code
	
	vec4 pr0 = texture2D(texPhi, vec2(tex.x, tex.y - gridSizeInv.y));
	vec4 pr1 = texture2D(texPhi, vec2(tex.x, tex.y));
	vec4 pr2 = texture2D(texPhi, vec2(tex.x, tex.y + gridSizeInv.y));
	vec4 pr3 = texture2D(texPhi, vec2(tex.x, tex.y + 2.0 * gridSizeInv.y));
	vec4 rows;
	rows.x = cubic(pr0, t.x);
	rows.y = cubic(pr1, t.x);
	rows.z = cubic(pr2, t.x);
	rows.w = cubic(pr3, t.x);
	return cubic(rows, t.y);
}

void main()
{
	vec3 uv_phi = texture2D(texUV, varyTexCoord).rgb;
	//float phi = uv_phi.b;
	float phi = phiBicubic();
	
	phi = phi * 5.0 - 4.0; // [-4:1]
	//gl_FragColor = vec4(phi, 0.0, 0.0, 1.0);
	//return;
	
	vec2 uv = uv_phi.rg;
	uv = abs(uv * 0.5 - 0.25); // <- MADD for: uv = (uv - vec2(0.5)) * 0.5;
	float speed = uv.x + uv.y;  // float speed = length(uv);
	speed = max(speed, 0.006);
	
	vec3 n = texture2D(texNormal, varyTexCoordNormal0).rgb - 0.5; // instead of usual *2 - 1, plus the following divide by two
	n += texture2D(texNormal, varyTexCoordNormal1).rgb - 0.5;
	//n = normalize(n);
	
	vec3 n2 = texture2D(texNormal, varyTexCoordNormal2).rgb * 2.0 - 1.0;
	
	vec2 refrOffset = n.xy * ( uRippleStrAndBackgrScale.x * speed + uRippleStrAndBackgrScale.y );
	//refrOffset = clamp(refrOffset, vec2(-0.02, -0.02), vec2(0.02, 0.02));
	float refrOffsetLen = length(refrOffset);	
	refrOffset = refrOffsetLen > 0.02 ? 	refrOffset * 0.02 / refrOffsetLen : refrOffset;
	
	vec4 backgrSample = texture2D(texBackgr, varyTexCoordBackgr);
	vec4 backgrSampleRefr = texture2D(texBackgr, varyTexCoordBackgr + refrOffset * uRippleStrAndBackgrScale.zw);
	//vec4 backgrSampleRefl = texture2D(texBackgr, varyTexCoord - phiGradient() * phi / 64.0 * 2.0);
	
	vec3 paintDensities = texture2D(texPaint, varyTexCoord).rgb;
	
	float blurWidth = uTransp_BlurWidth_SpecStrength.y;
	if(phi > blurWidth)
	{
		gl_FragColor = backgrSample;
	}
	else
	{
		
	
		// GOOD COLOR!  vec4(0.6, 0.85, 0.93, 1.0)
		//float t = clamp(-phi * 0.45, 0.7, 0.9);
		float t = smoothstep(0.0, 3.0, -phi);
		float transp = uTransp_BlurWidth_SpecStrength.x * (0.8 + 0.2 * t);
		gl_FragColor = backgrSampleRefr * transp + ( vec4(uWaterColor, 1.0) ) * (1.0 - transp);
		
		paintDensities *= t;
		vec3 paint = paintDensities.r * uPaintColor0 + paintDensities.g * uPaintColor1 + paintDensities.b * uPaintColor2;
		float paintAmount = min(paintDensities.r + paintDensities.g + paintDensities.b, 1.0);
		gl_FragColor.rgb = paint + (1.0 - paintAmount) * gl_FragColor.rgb;
		
		
		//t = smoothstep(0.0, 3.0, -phi);
		//t = t * 0.3 + 0.7;
		//gl_FragColor = gl_FragColor * t + backgrSampleRefl * (1.0 - t);
		
		t = clamp(-phi * 2.0, 0.0, 1.0);
		float specular = clamp(n2.x+n2.y, 0.0, 1.0);
		gl_FragColor.rgb += (1.0 - t) * specular * uTransp_BlurWidth_SpecStrength.z;
			
			
		if(phi > -blurWidth) // questionable if. Seems to help a bit on Ainol, not so much on SGS
		{
			t = smoothstep(-blurWidth, blurWidth, phi);
			gl_FragColor = (1.0-t) * gl_FragColor + (t) * backgrSample;
		}
		
		
		
	}
	
	
}
